# This file was auto-generated by Fern from our API Definition.

import typing

from .agent_v1settings_agent_context_messages_item_content import AgentV1SettingsAgentContextMessagesItemContent
from .agent_v1settings_agent_context_messages_item_function_calls import (
    AgentV1SettingsAgentContextMessagesItemFunctionCalls,
)

AgentV1SettingsAgentContextMessagesItem = typing.Union[
    AgentV1SettingsAgentContextMessagesItemContent, AgentV1SettingsAgentContextMessagesItemFunctionCalls
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ....types.think_settings_v1 import ThinkSettingsV1

AgentV1SettingsAgentThink = typing.Union[ThinkSettingsV1, typing.List[ThinkSettingsV1]]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .read_v1response_metadata import ReadV1ResponseMetadataParams
from .read_v1response_results import ReadV1ResponseResultsParams


class ReadV1ResponseParams(typing_extensions.TypedDict):
    """
    The standard text response
    """

    metadata: ReadV1ResponseMetadataParams
    results: ReadV1ResponseResultsParams



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from .agent_v1function_call_request_functions_item import AgentV1FunctionCallRequestFunctionsItemParams


class AgentV1FunctionCallRequestParams(typing_extensions.TypedDict):
    type: typing.Literal["FunctionCallRequest"]
    """
    Message type identifier for function call requests
    """

    functions: typing.Sequence[AgentV1FunctionCallRequestFunctionsItemParams]
    """
    Array of functions to be called
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from .list_project_keys_v1response_api_keys_item import ListProjectKeysV1ResponseApiKeysItemParams


class ListProjectKeysV1ResponseParams(typing_extensions.TypedDict):
    api_keys: typing_extensions.NotRequired[typing.Sequence[ListProjectKeysV1ResponseApiKeysItemParams]]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..types.listen_v1response_results_channels_item import ListenV1ResponseResultsChannelsItem

ListenV1ResponseResultsChannelsParams = typing.List[ListenV1ResponseResultsChannelsItem]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class SharedSentimentsSegmentsItemParams(typing_extensions.TypedDict):
    text: typing_extensions.NotRequired[str]
    start_word: typing_extensions.NotRequired[float]
    end_word: typing_extensions.NotRequired[float]
    sentiment: typing_extensions.NotRequired[str]
    sentiment_score: typing_extensions.NotRequired[float]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AgentV1SettingsAgentListenProviderV2(UniversalBaseModel):
    type: typing.Literal["deepgram"] = pydantic.Field(default="deepgram")
    """
    Provider type for speech-to-text
    """

    model: str = pydantic.Field()
    """
    Model to use for speech to text using the V2 API (e.g. flux-general-en)
    """

    keyterms: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Prompt keyterm recognition to improve Keyword Recall Rate
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.pydantic_utilities import parse_obj_as
from ....core.request_options import RequestOptions
from ....core.serialization import convert_and_respect_annotation_metadata
from ....errors.bad_request_error import BadRequestError
from ....requests.read_v1request import ReadV1RequestParams
from ....types.read_v1response import ReadV1Response
from .types.text_analyze_request_callback_method import TextAnalyzeRequestCallbackMethod
from .types.text_analyze_request_custom_intent_mode import TextAnalyzeRequestCustomIntentMode
from .types.text_analyze_request_custom_topic_mode import TextAnalyzeRequestCustomTopicMode
from .types.text_analyze_request_summarize import TextAnalyzeRequestSummarize

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTextClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def analyze(
        self,
        *,
        request: ReadV1RequestParams,
        callback: typing.Optional[str] = None,
        callback_method: typing.Optional[TextAnalyzeRequestCallbackMethod] = None,
        sentiment: typing.Optional[bool] = None,
        summarize: typing.Optional[TextAnalyzeRequestSummarize] = None,
        tag: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        topics: typing.Optional[bool] = None,
        custom_topic: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        custom_topic_mode: typing.Optional[TextAnalyzeRequestCustomTopicMode] = None,
        intents: typing.Optional[bool] = None,
        custom_intent: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        custom_intent_mode: typing.Optional[TextAnalyzeRequestCustomIntentMode] = None,
        language: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ReadV1Response]:
        """
        Analyze text content using Deepgrams text analysis API

        Parameters
        ----------
        request : ReadV1RequestParams

        callback : typing.Optional[str]
            URL to which we'll make the callback request

        callback_method : typing.Optional[TextAnalyzeRequestCallbackMethod]
            HTTP method by which the callback request will be made

        sentiment : typing.Optional[bool]
            Recognizes the sentiment throughout a transcript or text

        summarize : typing.Optional[TextAnalyzeRequestSummarize]
            Summarize content. For Listen API, supports string version option. For Read API, accepts boolean only.

        tag : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Label your requests for the purpose of identification during usage reporting

        topics : typing.Optional[bool]
            Detect topics throughout a transcript or text

        custom_topic : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Custom topics you want the model to detect within your input audio or text if present Submit up to `100`.

        custom_topic_mode : typing.Optional[TextAnalyzeRequestCustomTopicMode]
            Sets how the model will interpret strings submitted to the `custom_topic` param. When `strict`, the model will only return topics submitted using the `custom_topic` param. When `extended`, the model will return its own detected topics in addition to those submitted using the `custom_topic` param

        intents : typing.Optional[bool]
            Recognizes speaker intent throughout a transcript or text

        custom_intent : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Custom intents you want the model to detect within your input audio if present

        custom_intent_mode : typing.Optional[TextAnalyzeRequestCustomIntentMode]
            Sets how the model will interpret intents submitted to the `custom_intent` param. When `strict`, the model will only return intents submitted using the `custom_intent` param. When `extended`, the model will return its own detected intents in the `custom_intent` param.

        language : typing.Optional[str]
            The [BCP-47 language tag](https://tools.ietf.org/html/bcp47) that hints at the primary spoken language. Depending on the Model and API endpoint you choose only certain languages are available

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ReadV1Response]
            Successful text analysis
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/read",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            params={
                "callback": callback,
                "callback_method": callback_method,
                "sentiment": sentiment,
                "summarize": summarize,
                "tag": tag,
                "topics": topics,
                "custom_topic": custom_topic,
                "custom_topic_mode": custom_topic_mode,
                "intents": intents,
                "custom_intent": custom_intent,
                "custom_intent_mode": custom_intent_mode,
                "language": language,
            },
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=ReadV1RequestParams, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReadV1Response,
                    parse_obj_as(
                        type_=ReadV1Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTextClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def analyze(
        self,
        *,
        request: ReadV1RequestParams,
        callback: typing.Optional[str] = None,
        callback_method: typing.Optional[TextAnalyzeRequestCallbackMethod] = None,
        sentiment: typing.Optional[bool] = None,
        summarize: typing.Optional[TextAnalyzeRequestSummarize] = None,
        tag: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        topics: typing.Optional[bool] = None,
        custom_topic: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        custom_topic_mode: typing.Optional[TextAnalyzeRequestCustomTopicMode] = None,
        intents: typing.Optional[bool] = None,
        custom_intent: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        custom_intent_mode: typing.Optional[TextAnalyzeRequestCustomIntentMode] = None,
        language: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ReadV1Response]:
        """
        Analyze text content using Deepgrams text analysis API

        Parameters
        ----------
        request : ReadV1RequestParams

        callback : typing.Optional[str]
            URL to which we'll make the callback request

        callback_method : typing.Optional[TextAnalyzeRequestCallbackMethod]
            HTTP method by which the callback request will be made

        sentiment : typing.Optional[bool]
            Recognizes the sentiment throughout a transcript or text

        summarize : typing.Optional[TextAnalyzeRequestSummarize]
            Summarize content. For Listen API, supports string version option. For Read API, accepts boolean only.

        tag : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Label your requests for the purpose of identification during usage reporting

        topics : typing.Optional[bool]
            Detect topics throughout a transcript or text

        custom_topic : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Custom topics you want the model to detect within your input audio or text if present Submit up to `100`.

        custom_topic_mode : typing.Optional[TextAnalyzeRequestCustomTopicMode]
            Sets how the model will interpret strings submitted to the `custom_topic` param. When `strict`, the model will only return topics submitted using the `custom_topic` param. When `extended`, the model will return its own detected topics in addition to those submitted using the `custom_topic` param

        intents : typing.Optional[bool]
            Recognizes speaker intent throughout a transcript or text

        custom_intent : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Custom intents you want the model to detect within your input audio if present

        custom_intent_mode : typing.Optional[TextAnalyzeRequestCustomIntentMode]
            Sets how the model will interpret intents submitted to the `custom_intent` param. When `strict`, the model will only return intents submitted using the `custom_intent` param. When `extended`, the model will return its own detected intents in the `custom_intent` param.

        language : typing.Optional[str]
            The [BCP-47 language tag](https://tools.ietf.org/html/bcp47) that hints at the primary spoken language. Depending on the Model and API endpoint you choose only certain languages are available

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ReadV1Response]
            Successful text analysis
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/read",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            params={
                "callback": callback,
                "callback_method": callback_method,
                "sentiment": sentiment,
                "summarize": summarize,
                "tag": tag,
                "topics": topics,
                "custom_topic": custom_topic,
                "custom_topic_mode": custom_topic_mode,
                "intents": intents,
                "custom_intent": custom_intent,
                "custom_intent_mode": custom_intent_mode,
                "language": language,
            },
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=ReadV1RequestParams, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReadV1Response,
                    parse_obj_as(
                        type_=ReadV1Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class UsageFieldsV1ResponseModelsItem(UniversalBaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of the model.
    """

    language: typing.Optional[str] = pydantic.Field(default=None)
    """
    The language supported by the model (IETF language tag).
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    Version identifier of the model, typically with a date and a revision number.
    """

    model_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the model.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .get_project_distribution_credentials_v1response_distribution_credentials import (
    GetProjectDistributionCredentialsV1ResponseDistributionCredentialsParams,
)
from .get_project_distribution_credentials_v1response_member import (
    GetProjectDistributionCredentialsV1ResponseMemberParams,
)


class GetProjectDistributionCredentialsV1ResponseParams(typing_extensions.TypedDict):
    member: GetProjectDistributionCredentialsV1ResponseMemberParams
    distribution_credentials: GetProjectDistributionCredentialsV1ResponseDistributionCredentialsParams



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class SpeakSettingsV1EndpointParams(typing_extensions.TypedDict):
    """
    Optional if provider is Deepgram. Required for non-Deepgram TTS providers.
    When present, must include url field and headers object. Valid schemes are https and wss with wss only supported for Eleven Labs.
    """

    url: typing_extensions.NotRequired[str]
    """
    Custom TTS endpoint URL. Cannot contain `output_format` or `model_id` query parameters when the provider is Eleven Labs.
    """

    headers: typing_extensions.NotRequired[typing.Dict[str, str]]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class AgentV1AgentAudioDoneParams(typing_extensions.TypedDict):
    type: typing.Literal["AgentAudioDone"]
    """
    Message type identifier indicating the agent has finished sending audio
    """



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AgentV1SettingsAgentListenProvider_V1(UniversalBaseModel):
    version: typing.Literal["v1"] = "v1"
    type: typing.Literal["deepgram"] = "deepgram"
    model: typing.Optional[str] = None
    language: typing.Optional[str] = None
    keyterms: typing.Optional[typing.List[str]] = None
    smart_format: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class AgentV1SettingsAgentListenProvider_V2(UniversalBaseModel):
    version: typing.Literal["v2"] = "v2"
    type: typing.Literal["deepgram"] = "deepgram"
    model: str
    keyterms: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


AgentV1SettingsAgentListenProvider = typing_extensions.Annotated[
    typing.Union[AgentV1SettingsAgentListenProvider_V1, AgentV1SettingsAgentListenProvider_V2],
    pydantic.Field(discriminator="version"),
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .listen_v1response_results_channels import ListenV1ResponseResultsChannelsParams
from .listen_v1response_results_summary import ListenV1ResponseResultsSummaryParams
from .listen_v1response_results_utterances import ListenV1ResponseResultsUtterancesParams
from .shared_intents import SharedIntentsParams
from .shared_sentiments import SharedSentimentsParams
from .shared_topics import SharedTopicsParams


class ListenV1ResponseResultsParams(typing_extensions.TypedDict):
    channels: ListenV1ResponseResultsChannelsParams
    utterances: typing_extensions.NotRequired[ListenV1ResponseResultsUtterancesParams]
    summary: typing_extensions.NotRequired[ListenV1ResponseResultsSummaryParams]
    topics: typing_extensions.NotRequired[SharedTopicsParams]
    intents: typing_extensions.NotRequired[SharedIntentsParams]
    sentiments: typing_extensions.NotRequired[SharedSentimentsParams]



# This file was auto-generated by Fern from our API Definition.

import typing

AwsPollySpeakProviderVoice = typing.Union[
    typing.Literal["Matthew", "Joanna", "Amy", "Emma", "Brian", "Arthur", "Aria", "Ayanda"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt

import typing_extensions


class ListProjectPurchasesV1ResponseOrdersItemParams(typing_extensions.TypedDict):
    order_id: typing_extensions.NotRequired[str]
    expiration: typing_extensions.NotRequired[dt.datetime]
    created: typing_extensions.NotRequired[dt.datetime]
    amount: typing_extensions.NotRequired[float]
    units: typing_extensions.NotRequired[str]
    order_type: typing_extensions.NotRequired[str]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from .think_settings_v1context_length import ThinkSettingsV1ContextLengthParams
from .think_settings_v1endpoint import ThinkSettingsV1EndpointParams
from .think_settings_v1functions_item import ThinkSettingsV1FunctionsItemParams
from .think_settings_v1provider import ThinkSettingsV1ProviderParams


class ThinkSettingsV1Params(typing_extensions.TypedDict):
    provider: ThinkSettingsV1ProviderParams
    endpoint: typing_extensions.NotRequired[ThinkSettingsV1EndpointParams]
    """
    Optional for non-Deepgram LLM providers. When present, must include url field and headers object
    """

    functions: typing_extensions.NotRequired[typing.Sequence[ThinkSettingsV1FunctionsItemParams]]
    prompt: typing_extensions.NotRequired[str]
    context_length: typing_extensions.NotRequired[ThinkSettingsV1ContextLengthParams]
    """
    Specifies the number of characters retained in context between user messages, agent responses, and function calls. This setting is only configurable when a custom think endpoint is used
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class GrantV1ResponseParams(typing_extensions.TypedDict):
    access_token: str
    """
    JSON Web Token (JWT)
    """

    expires_in: typing_extensions.NotRequired[float]
    """
    Time in seconds until the JWT expires
    """



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.datetime_utils import serialize_datetime
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from .....errors.bad_request_error import BadRequestError
from .....types.get_project_request_v1response import GetProjectRequestV1Response
from .....types.list_project_requests_v1response import ListProjectRequestsV1Response
from .types.requests_list_request_deployment import RequestsListRequestDeployment
from .types.requests_list_request_endpoint import RequestsListRequestEndpoint
from .types.requests_list_request_method import RequestsListRequestMethod
from .types.requests_list_request_status import RequestsListRequestStatus


class RawRequestsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        project_id: str,
        *,
        start: typing.Optional[dt.datetime] = None,
        end: typing.Optional[dt.datetime] = None,
        limit: typing.Optional[float] = None,
        page: typing.Optional[float] = None,
        accessor: typing.Optional[str] = None,
        request_id: typing.Optional[str] = None,
        deployment: typing.Optional[RequestsListRequestDeployment] = None,
        endpoint: typing.Optional[RequestsListRequestEndpoint] = None,
        method: typing.Optional[RequestsListRequestMethod] = None,
        status: typing.Optional[RequestsListRequestStatus] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ListProjectRequestsV1Response]:
        """
        Generates a list of requests for a specific project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        start : typing.Optional[dt.datetime]
            Start date of the requested date range. Formats accepted are YYYY-MM-DD, YYYY-MM-DDTHH:MM:SS, or YYYY-MM-DDTHH:MM:SS+HH:MM

        end : typing.Optional[dt.datetime]
            End date of the requested date range. Formats accepted are YYYY-MM-DD, YYYY-MM-DDTHH:MM:SS, or YYYY-MM-DDTHH:MM:SS+HH:MM

        limit : typing.Optional[float]
            Number of results to return per page. Default 10. Range [1,1000]

        page : typing.Optional[float]
            Navigate and return the results to retrieve specific portions of information of the response

        accessor : typing.Optional[str]
            Filter for requests where a specific accessor was used

        request_id : typing.Optional[str]
            Filter for a specific request id

        deployment : typing.Optional[RequestsListRequestDeployment]
            Filter for requests where a specific deployment was used

        endpoint : typing.Optional[RequestsListRequestEndpoint]
            Filter for requests where a specific endpoint was used

        method : typing.Optional[RequestsListRequestMethod]
            Filter for requests where a specific method was used

        status : typing.Optional[RequestsListRequestStatus]
            Filter for requests that succeeded (status code < 300) or failed (status code >=400)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListProjectRequestsV1Response]
            A list of requests for a specific project
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/requests",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "start": serialize_datetime(start) if start is not None else None,
                "end": serialize_datetime(end) if end is not None else None,
                "limit": limit,
                "page": page,
                "accessor": accessor,
                "request_id": request_id,
                "deployment": deployment,
                "endpoint": endpoint,
                "method": method,
                "status": status,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListProjectRequestsV1Response,
                    parse_obj_as(
                        type_=ListProjectRequestsV1Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, project_id: str, request_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetProjectRequestV1Response]:
        """
        Retrieves a specific request for a specific project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        request_id : str
            The unique identifier of the request

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetProjectRequestV1Response]
            A specific request for a specific project
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/requests/{jsonable_encoder(request_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetProjectRequestV1Response,
                    parse_obj_as(
                        type_=GetProjectRequestV1Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawRequestsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        project_id: str,
        *,
        start: typing.Optional[dt.datetime] = None,
        end: typing.Optional[dt.datetime] = None,
        limit: typing.Optional[float] = None,
        page: typing.Optional[float] = None,
        accessor: typing.Optional[str] = None,
        request_id: typing.Optional[str] = None,
        deployment: typing.Optional[RequestsListRequestDeployment] = None,
        endpoint: typing.Optional[RequestsListRequestEndpoint] = None,
        method: typing.Optional[RequestsListRequestMethod] = None,
        status: typing.Optional[RequestsListRequestStatus] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ListProjectRequestsV1Response]:
        """
        Generates a list of requests for a specific project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        start : typing.Optional[dt.datetime]
            Start date of the requested date range. Formats accepted are YYYY-MM-DD, YYYY-MM-DDTHH:MM:SS, or YYYY-MM-DDTHH:MM:SS+HH:MM

        end : typing.Optional[dt.datetime]
            End date of the requested date range. Formats accepted are YYYY-MM-DD, YYYY-MM-DDTHH:MM:SS, or YYYY-MM-DDTHH:MM:SS+HH:MM

        limit : typing.Optional[float]
            Number of results to return per page. Default 10. Range [1,1000]

        page : typing.Optional[float]
            Navigate and return the results to retrieve specific portions of information of the response

        accessor : typing.Optional[str]
            Filter for requests where a specific accessor was used

        request_id : typing.Optional[str]
            Filter for a specific request id

        deployment : typing.Optional[RequestsListRequestDeployment]
            Filter for requests where a specific deployment was used

        endpoint : typing.Optional[RequestsListRequestEndpoint]
            Filter for requests where a specific endpoint was used

        method : typing.Optional[RequestsListRequestMethod]
            Filter for requests where a specific method was used

        status : typing.Optional[RequestsListRequestStatus]
            Filter for requests that succeeded (status code < 300) or failed (status code >=400)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListProjectRequestsV1Response]
            A list of requests for a specific project
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/requests",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "start": serialize_datetime(start) if start is not None else None,
                "end": serialize_datetime(end) if end is not None else None,
                "limit": limit,
                "page": page,
                "accessor": accessor,
                "request_id": request_id,
                "deployment": deployment,
                "endpoint": endpoint,
                "method": method,
                "status": status,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListProjectRequestsV1Response,
                    parse_obj_as(
                        type_=ListProjectRequestsV1Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, project_id: str, request_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetProjectRequestV1Response]:
        """
        Retrieves a specific request for a specific project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        request_id : str
            The unique identifier of the request

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetProjectRequestV1Response]
            A specific request for a specific project
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/projects/{jsonable_encoder(project_id)}/requests/{jsonable_encoder(request_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetProjectRequestV1Response,
                    parse_obj_as(
                        type_=GetProjectRequestV1Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class CartesiaSpeakProviderVoiceParams(typing_extensions.TypedDict):
    mode: str
    """
    Cartesia voice mode
    """

    id: str
    """
    Cartesia voice ID
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import jsonable_encoder
from ....core.pydantic_utilities import parse_obj_as
from ....core.request_options import RequestOptions
from ....errors.bad_request_error import BadRequestError
from ....types.get_model_v1response import GetModelV1Response
from ....types.list_models_v1response import ListModelsV1Response


class RawModelsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, include_outdated: typing.Optional[bool] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ListModelsV1Response]:
        """
        Returns metadata on all the latest public models. To retrieve custom models, use Get Project Models.

        Parameters
        ----------
        include_outdated : typing.Optional[bool]
            returns non-latest versions of models

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListModelsV1Response]
            A list of models
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/models",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "include_outdated": include_outdated,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListModelsV1Response,
                    parse_obj_as(
                        type_=ListModelsV1Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, model_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetModelV1Response]:
        """
        Returns metadata for a specific public model

        Parameters
        ----------
        model_id : str
            The specific UUID of the model

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetModelV1Response]
            A model object that can be either STT or TTS
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/models/{jsonable_encoder(model_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetModelV1Response,
                    parse_obj_as(
                        type_=GetModelV1Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawModelsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, include_outdated: typing.Optional[bool] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ListModelsV1Response]:
        """
        Returns metadata on all the latest public models. To retrieve custom models, use Get Project Models.

        Parameters
        ----------
        include_outdated : typing.Optional[bool]
            returns non-latest versions of models

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListModelsV1Response]
            A list of models
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/models",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "include_outdated": include_outdated,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListModelsV1Response,
                    parse_obj_as(
                        type_=ListModelsV1Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, model_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetModelV1Response]:
        """
        Returns metadata for a specific public model

        Parameters
        ----------
        model_id : str
            The specific UUID of the model

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetModelV1Response]
            A model object that can be either STT or TTS
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/models/{jsonable_encoder(model_id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetModelV1Response,
                    parse_obj_as(
                        type_=GetModelV1Response,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



"""Interface definitions for custom WebSocket transports.

Implement one of these protocols to provide a custom transport to the
Deepgram SDK via the ``transport_factory`` parameter::

    from deepgram import DeepgramClient
    from deepgram.transport_interface import SyncTransport

    class MyTransport:
        \"\"\"Custom transport — must satisfy SyncTransport.\"\"\"

        def __init__(self, url: str, headers: dict):
            ...  # establish your connection here

        def send(self, data):
            ...  # send text (str) or binary (bytes) data

        def recv(self):
            ...  # return the next message (str or bytes)

        def __iter__(self):
            ...  # yield messages until the connection closes

        def close(self):
            ...  # tear down the connection

    client = DeepgramClient(api_key="...", transport_factory=MyTransport)

The transport factory is the **class itself** (or any callable). The SDK
calls ``factory(url, headers)`` and gets back a transport object. The SDK
handles context-manager wrapping and calls ``close()`` automatically.
"""

from typing import Any, Iterator

from typing_extensions import Protocol, runtime_checkable


@runtime_checkable
class SyncTransport(Protocol):
    """Protocol that sync transport objects must satisfy.

    Methods
    -------
    send(data)
        Send text (``str``) or binary (``bytes``) data over the connection.
    recv()
        Block until the next message arrives and return it.
    __iter__()
        Yield messages until the connection closes.
    close()
        Tear down the connection and release resources.
    """

    def send(self, data: Any) -> None: ...
    def recv(self) -> Any: ...
    def __iter__(self) -> Iterator: ...
    def close(self) -> None: ...


@runtime_checkable
class AsyncTransport(Protocol):
    """Protocol that async transport objects must satisfy.

    Methods
    -------
    send(data)
        Send text (``str``) or binary (``bytes``) data over the connection.
    recv()
        Await the next message and return it.
    __aiter__()
        Async-yield messages until the connection closes.
    close()
        Tear down the connection and release resources.
    """

    async def send(self, data: Any) -> None: ...
    async def recv(self) -> Any: ...
    def __aiter__(self) -> Any: ...
    async def close(self) -> None: ...



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .listen_v1results_metadata_model_info import ListenV1ResultsMetadataModelInfo


class ListenV1ResultsMetadata(UniversalBaseModel):
    request_id: str = pydantic.Field()
    """
    The request ID
    """

    model_info: ListenV1ResultsMetadataModelInfo
    model_uuid: str = pydantic.Field()
    """
    The model UUID
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



"""Concrete transport implementations for the Deepgram Python SDK.

Custom transports can be passed to ``AsyncDeepgramClient(transport_factory=...)``
or ``DeepgramClient(transport_factory=...)``.

See ``deepgram.transport_interface`` for the protocol definitions.
"""

__all__: list = []



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.speak_v1close_type import SpeakV1CloseType


class SpeakV1CloseParams(typing_extensions.TypedDict):
    type: SpeakV1CloseType
    """
    Message type identifier
    """



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions
from ..types.anthropic_think_provider_model import AnthropicThinkProviderModel
from ..types.aws_bedrock_think_provider_model import AwsBedrockThinkProviderModel
from ..types.google_think_provider_model import GoogleThinkProviderModel
from ..types.open_ai_think_provider_model import OpenAiThinkProviderModel
from .aws_bedrock_think_provider_credentials import AwsBedrockThinkProviderCredentialsParams


class ThinkSettingsV1Provider_OpenAiParams(typing_extensions.TypedDict):
    type: typing.Literal["open_ai"]
    version: typing_extensions.NotRequired[typing.Literal["v1"]]
    model: OpenAiThinkProviderModel
    temperature: typing_extensions.NotRequired[float]


class ThinkSettingsV1Provider_AwsBedrockParams(typing_extensions.TypedDict):
    type: typing.Literal["aws_bedrock"]
    model: AwsBedrockThinkProviderModel
    temperature: typing_extensions.NotRequired[float]
    credentials: typing_extensions.NotRequired[AwsBedrockThinkProviderCredentialsParams]


class ThinkSettingsV1Provider_AnthropicParams(typing_extensions.TypedDict):
    type: typing.Literal["anthropic"]
    version: typing_extensions.NotRequired[typing.Literal["v1"]]
    model: AnthropicThinkProviderModel
    temperature: typing_extensions.NotRequired[float]


class ThinkSettingsV1Provider_GoogleParams(typing_extensions.TypedDict):
    type: typing.Literal["google"]
    version: typing_extensions.NotRequired[typing.Literal["v1beta"]]
    model: GoogleThinkProviderModel
    temperature: typing_extensions.NotRequired[float]


class ThinkSettingsV1Provider_GroqParams(typing_extensions.TypedDict):
    type: typing.Literal["groq"]
    version: typing_extensions.NotRequired[typing.Literal["v1"]]
    model: typing.Literal["openai/gpt-oss-20b"]
    temperature: typing_extensions.NotRequired[float]


ThinkSettingsV1ProviderParams = typing.Union[
    ThinkSettingsV1Provider_OpenAiParams,
    ThinkSettingsV1Provider_AwsBedrockParams,
    ThinkSettingsV1Provider_AnthropicParams,
    ThinkSettingsV1Provider_GoogleParams,
    ThinkSettingsV1Provider_GroqParams,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .list_project_distribution_credentials_v1response_distribution_credentials_item_distribution_credentials import (
    ListProjectDistributionCredentialsV1ResponseDistributionCredentialsItemDistributionCredentials,
)
from .list_project_distribution_credentials_v1response_distribution_credentials_item_member import (
    ListProjectDistributionCredentialsV1ResponseDistributionCredentialsItemMember,
)


class ListProjectDistributionCredentialsV1ResponseDistributionCredentialsItem(UniversalBaseModel):
    member: ListProjectDistributionCredentialsV1ResponseDistributionCredentialsItemMember
    distribution_credentials: (
        ListProjectDistributionCredentialsV1ResponseDistributionCredentialsItemDistributionCredentials
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ReadV1RequestTextParams(typing_extensions.TypedDict):
    text: str
    """
    The plain text to analyze
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from .listen_v1results_channel_alternatives_item_words_item import ListenV1ResultsChannelAlternativesItemWordsItemParams


class ListenV1ResultsChannelAlternativesItemParams(typing_extensions.TypedDict):
    transcript: str
    """
    The transcript of the transcription
    """

    confidence: float
    """
    The confidence of the transcription
    """

    languages: typing_extensions.NotRequired[typing.Sequence[str]]
    words: typing.Sequence[ListenV1ResultsChannelAlternativesItemWordsItemParams]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..types.anthropic_think_provider_model import AnthropicThinkProviderModel


class AnthropicParams(typing_extensions.TypedDict):
    type: typing.Literal["anthropic"]
    version: typing_extensions.NotRequired[typing.Literal["v1"]]
    """
    The REST API version for the Anthropic Messages API
    """

    model: AnthropicThinkProviderModel
    """
    Anthropic model to use
    """

    temperature: typing_extensions.NotRequired[float]
    """
    Anthropic temperature (0-1)
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ListenV1ResponseResultsChannelsItemAlternativesItemParagraphsParagraphsItemSentencesItemParams(
    typing_extensions.TypedDict
):
    text: typing_extensions.NotRequired[str]
    start: typing_extensions.NotRequired[float]
    end: typing_extensions.NotRequired[float]



"""Custom WebSocket transport support for the Deepgram Python SDK.

Allows users to swap in custom transports (BiDi/SSE, test doubles, proxied
connections) by providing a ``transport_factory`` when constructing a
:class:`DeepgramClient` or :class:`AsyncDeepgramClient`.

A transport factory is a callable: ``factory(url, headers)`` that returns a
context manager (sync) or async context manager (async) yielding a transport
object with ``send()``, ``recv()``, and iteration support.

Usage::

    from deepgram import DeepgramClient

    client = DeepgramClient(api_key="...", transport_factory=MySyncTransport)
"""

import importlib
import sys
from contextlib import asynccontextmanager, contextmanager
from typing import Any, Callable, Dict, Optional

from .transport_interface import AsyncTransport, SyncTransport

# Re-export so existing ``from deepgram.transport import SyncTransport`` still works.
__all__ = ["SyncTransport", "AsyncTransport", "install_transport", "restore_transport"]

# ---------------------------------------------------------------------------
# Module paths that contain the websocket references we need to patch.
# All 8 are auto-generated by Fern — we never modify their source.
# ---------------------------------------------------------------------------
_TARGET_MODULES = [
    "deepgram.listen.v1.raw_client",
    "deepgram.listen.v1.client",
    "deepgram.listen.v2.raw_client",
    "deepgram.listen.v2.client",
    "deepgram.speak.v1.raw_client",
    "deepgram.speak.v1.client",
    "deepgram.agent.v1.raw_client",
    "deepgram.agent.v1.client",
]

# Originals stashed by install_transport() so restore_transport() can undo.
_originals: Dict[str, Dict[str, Any]] = {}

# Active factories — used to detect conflicting installs.
_active_sync_factory: Optional[Callable] = None
_active_async_factory: Optional[Callable] = None


class _SyncTransportShim:
    """Drop-in replacement for ``websockets.sync.client``.

    The auto-generated code calls::

        websockets_sync_client.connect(ws_url, additional_headers=headers)

    This shim's ``.connect()`` delegates to the user-supplied factory.
    """

    def __init__(self, factory: Callable) -> None:
        self._factory = factory

    @contextmanager
    def connect(self, url: str, additional_headers: Optional[Dict[str, str]] = None):
        headers = additional_headers or {}
        transport = self._factory(url, headers)
        try:
            yield transport
        finally:
            transport.close()


class _AsyncTransportShim:
    """Drop-in replacement for ``websockets_client_connect``.

    The auto-generated code calls::

        websockets_client_connect(ws_url, extra_headers=headers)

    This shim is callable and returns an async context manager.
    """

    def __init__(self, factory: Callable) -> None:
        self._factory = factory

    @asynccontextmanager
    async def _connect(self, url: str, extra_headers: Optional[Dict[str, str]] = None):
        headers = extra_headers or {}
        transport = self._factory(url, headers)
        try:
            yield transport
        finally:
            await transport.close()

    def __call__(self, url: str, extra_headers: Optional[Dict[str, str]] = None):
        return self._connect(url, extra_headers=extra_headers)


def install_transport(
    *,
    sync_factory: Optional[Callable] = None,
    async_factory: Optional[Callable] = None,
) -> None:
    """Monkey-patch the 8 auto-generated modules to use custom transports.

    Parameters
    ----------
    sync_factory
        ``factory(url, headers) -> transport`` used for sync WebSocket clients.
    async_factory
        ``factory(url, headers) -> transport`` used for async WebSocket clients.

    Raises
    ------
    RuntimeError
        If a different transport factory is already installed. Re-installing
        the same factory is allowed (idempotent). Call :func:`restore_transport`
        first to switch factories.
    """
    global _active_sync_factory, _active_async_factory

    if sync_factory is not None and _active_sync_factory is not None and _active_sync_factory is not sync_factory:
        raise RuntimeError(
            "A different sync transport factory is already installed. "
            "Only one transport factory per process is supported because "
            "transport patching is global. Call restore_transport() before "
            "creating a new client with a different transport_factory."
        )
    if async_factory is not None and _active_async_factory is not None and _active_async_factory is not async_factory:
        raise RuntimeError(
            "A different async transport factory is already installed. "
            "Only one transport factory per process is supported because "
            "transport patching is global. Call restore_transport() before "
            "creating a new client with a different transport_factory."
        )

    if sync_factory is not None:
        _active_sync_factory = sync_factory
    if async_factory is not None:
        _active_async_factory = async_factory

    sync_shim = _SyncTransportShim(sync_factory) if sync_factory else None
    async_shim = _AsyncTransportShim(async_factory) if async_factory else None

    for mod_path in _TARGET_MODULES:
        mod = sys.modules.get(mod_path)
        if mod is None:
            try:
                mod = importlib.import_module(mod_path)
            except ImportError:
                continue

        # Stash originals on first install (don't overwrite if already stashed).
        if mod_path not in _originals:
            _originals[mod_path] = {
                "websockets_sync_client": getattr(mod, "websockets_sync_client", None),
                "websockets_client_connect": getattr(mod, "websockets_client_connect", None),
            }

        if sync_shim is not None and hasattr(mod, "websockets_sync_client"):
            mod.websockets_sync_client = sync_shim  # type: ignore[attr-defined]

        if async_shim is not None and hasattr(mod, "websockets_client_connect"):
            mod.websockets_client_connect = async_shim  # type: ignore[attr-defined]


def restore_transport() -> None:
    """Undo all patches applied by :func:`install_transport`."""
    global _active_sync_factory, _active_async_factory

    for mod_path, saved in _originals.items():
        mod = sys.modules.get(mod_path)
        if mod is None:
            continue
        for attr, original in saved.items():
            if original is not None:
                setattr(mod, attr, original)
    _originals.clear()
    _active_sync_factory = None
    _active_async_factory = None



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .open_ai_think_provider_model import OpenAiThinkProviderModel


class OpenAiThinkProvider(UniversalBaseModel):
    type: typing.Literal["open_ai"] = "open_ai"
    version: typing.Optional[typing.Literal["v1"]] = pydantic.Field(default=None)
    """
    The REST API version for the OpenAI chat completions API
    """

    model: OpenAiThinkProviderModel = pydantic.Field()
    """
    OpenAI model to use
    """

    temperature: typing.Optional[float] = pydantic.Field(default=None)
    """
    OpenAI temperature (0-2)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ListenV2TurnInfoEvent = typing.Union[
    typing.Literal["Update", "StartOfTurn", "EagerEndOfTurn", "TurnResumed", "EndOfTurn"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .read_v1response_metadata_metadata_intents_info import ReadV1ResponseMetadataMetadataIntentsInfo
from .read_v1response_metadata_metadata_sentiment_info import ReadV1ResponseMetadataMetadataSentimentInfo
from .read_v1response_metadata_metadata_summary_info import ReadV1ResponseMetadataMetadataSummaryInfo
from .read_v1response_metadata_metadata_topics_info import ReadV1ResponseMetadataMetadataTopicsInfo


class ReadV1ResponseMetadataMetadata(UniversalBaseModel):
    request_id: typing.Optional[str] = None
    created: typing.Optional[dt.datetime] = None
    language: typing.Optional[str] = None
    summary_info: typing.Optional[ReadV1ResponseMetadataMetadataSummaryInfo] = None
    sentiment_info: typing.Optional[ReadV1ResponseMetadataMetadataSentimentInfo] = None
    topics_info: typing.Optional[ReadV1ResponseMetadataMetadataTopicsInfo] = None
    intents_info: typing.Optional[ReadV1ResponseMetadataMetadataIntentsInfo] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class SpeakV1WarningParams(typing_extensions.TypedDict):
    type: typing.Literal["Warning"]
    """
    Message type identifier
    """

    description: str
    """
    A description of what went wrong
    """

    code: str
    """
    Error code identifying the type of error
    """



"""
Custom client entrypoints that extend the generated BaseClient/AsyncBaseClient.

Adds support for:
- `access_token` as an alternative to `api_key` with the following rules:
  - If `access_token` is provided, it takes precedence and sets `Authorization: bearer <token>`
  - When `access_token` is used, `api_key` is forced to "token" to satisfy the generator,
    but the Authorization header is overridden for all HTTP and WebSocket requests.
- `session_id` as a header sent with every request and websocket connection:
  - If `session_id` is provided, it will be used; otherwise, a UUID is auto-generated
  - The session_id is sent as the `x-deepgram-session-id` header
- `transport_factory` to replace the default `websockets` transport with a custom one:
  - A callable: ``factory(url, headers) -> transport`` returning an object with
    ``send()``, ``recv()``, iteration, and ``close()`` support.
"""

import types
import uuid
from typing import Any, Callable, Dict, Optional

from .base_client import AsyncBaseClient, BaseClient
from .transport import install_transport

from deepgram.core.client_wrapper import BaseClientWrapper


def _apply_bearer_authorization_override(client_wrapper: BaseClientWrapper, bearer_token: str) -> None:
    """Override header providers to always use a Bearer authorization token.

    This updates both:
    - client_wrapper.get_headers() used by WebSocket clients
    - client_wrapper.httpx_client.base_headers used by HTTP clients
    """
    original_get_headers = client_wrapper.get_headers

    def _get_headers_with_bearer(_self: Any) -> Dict[str, str]:
        headers = original_get_headers()
        headers["Authorization"] = f"bearer {bearer_token}"
        return headers

    # Override on wrapper for WebSockets
    client_wrapper.get_headers = types.MethodType(_get_headers_with_bearer, client_wrapper)  # type: ignore[method-assign]

    # Override on HTTP client for REST requests
    if hasattr(client_wrapper, "httpx_client") and hasattr(client_wrapper.httpx_client, "base_headers"):
        client_wrapper.httpx_client.base_headers = client_wrapper.get_headers


class DeepgramClient(BaseClient):
    """
    Custom Deepgram client that extends the generated BaseClient.

    Supports:
    - `session_id`: Optional session identifier. If not provided, a UUID is auto-generated.
                     Sent as `x-deepgram-session-id` header in all requests and websocket connections.
    - `access_token`: Alternative to `api_key`. If provided, uses Bearer token authentication.
    - `transport_factory`: Custom sync WebSocket transport factory. A callable
                           ``factory(url, headers) -> transport`` whose return value must support
                           ``send()``, ``recv()``, iteration, and ``close()``.
    - `telemetry_opt_out`: Telemetry opt-out flag (maintained for backwards compatibility, no-op).
    - `telemetry_handler`: Telemetry handler (maintained for backwards compatibility, no-op).
    """

    def __init__(self, *args, **kwargs) -> None:
        access_token: Optional[str] = kwargs.pop("access_token", None)
        session_id: Optional[str] = kwargs.pop("session_id", None)
        transport_factory: Optional[Callable] = kwargs.pop("transport_factory", None)
        telemetry_opt_out: bool = bool(kwargs.pop("telemetry_opt_out", True))
        telemetry_handler: Optional[Any] = kwargs.pop("telemetry_handler", None)

        # Use provided session_id or generate one
        final_session_id = session_id if session_id is not None else str(uuid.uuid4())

        # Ensure headers object exists for pass-through custom headers
        headers: Optional[Dict[str, str]] = kwargs.get("headers")
        if headers is None:
            headers = {}
            kwargs["headers"] = headers

        # Ensure every request has a session identifier header
        headers["x-deepgram-session-id"] = final_session_id

        # Handle access_token: if provided, it takes precedence over api_key
        # The base client requires api_key, so we set a placeholder if needed
        # The Authorization header will be overridden to use Bearer token
        if access_token is not None:
            # Set a placeholder api_key if none provided (base client requires it)
            if kwargs.get("api_key") is None:
                kwargs["api_key"] = "token"

        super().__init__(*args, **kwargs)
        self.session_id = final_session_id

        # Override Authorization header to use Bearer token if access_token was provided
        if access_token is not None:
            _apply_bearer_authorization_override(self._client_wrapper, access_token)

        # Install custom WebSocket transport if provided
        if transport_factory is not None:
            install_transport(sync_factory=transport_factory)

        # Store telemetry handler for backwards compatibility (no-op, telemetry not implemented)
        self._telemetry_handler = None


class AsyncDeepgramClient(AsyncBaseClient):
    """
    Custom async Deepgram client that extends the generated AsyncBaseClient.

    Supports:
    - `session_id`: Optional session identifier. If not provided, a UUID is auto-generated.
                     Sent as `x-deepgram-session-id` header in all requests and websocket connections.
    - `access_token`: Alternative to `api_key`. If provided, uses Bearer token authentication.
    - `transport_factory`: Custom async WebSocket transport factory. A callable
                           ``factory(url, headers) -> transport`` whose return value must support
                           ``send()``, ``recv()``, async iteration, and ``close()``.
    - `telemetry_opt_out`: Telemetry opt-out flag (maintained for backwards compatibility, no-op).
    - `telemetry_handler`: Telemetry handler (maintained for backwards compatibility, no-op).
    """

    def __init__(self, *args, **kwargs) -> None:
        access_token: Optional[str] = kwargs.pop("access_token", None)
        session_id: Optional[str] = kwargs.pop("session_id", None)
        transport_factory: Optional[Callable] = kwargs.pop("transport_factory", None)
        telemetry_opt_out: bool = bool(kwargs.pop("telemetry_opt_out", True))
        telemetry_handler: Optional[Any] = kwargs.pop("telemetry_handler", None)

        # Use provided session_id or generate one
        final_session_id = session_id if session_id is not None else str(uuid.uuid4())

        # Ensure headers object exists for pass-through custom headers
        headers: Optional[Dict[str, str]] = kwargs.get("headers")
        if headers is None:
            headers = {}
            kwargs["headers"] = headers

        # Ensure every request has a session identifier header
        headers["x-deepgram-session-id"] = final_session_id

        # Handle access_token: if provided, it takes precedence over api_key
        # The base client requires api_key, so we set a placeholder if needed
        # The Authorization header will be overridden to use Bearer token
        if access_token is not None:
            # Set a placeholder api_key if none provided (base client requires it)
            if kwargs.get("api_key") is None:
                kwargs["api_key"] = "token"

        super().__init__(*args, **kwargs)
        self.session_id = final_session_id

        # Override Authorization header to use Bearer token if access_token was provided
        if access_token is not None:
            _apply_bearer_authorization_override(self._client_wrapper, access_token)

        # Install custom WebSocket transport if provided
        if transport_factory is not None:
            install_transport(async_factory=transport_factory)

        # Store telemetry handler for backwards compatibility (no-op, telemetry not implemented)
        self._telemetry_handler = None



# This file was auto-generated by Fern from our API Definition.

import typing

from .....requests.listen_v1accepted_response import ListenV1AcceptedResponseParams
from .....requests.listen_v1response import ListenV1ResponseParams

MediaTranscribeResponseParams = typing.Union[ListenV1ResponseParams, ListenV1AcceptedResponseParams]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from .think_settings_v1functions_item_endpoint import ThinkSettingsV1FunctionsItemEndpointParams


class ThinkSettingsV1FunctionsItemParams(typing_extensions.TypedDict):
    name: typing_extensions.NotRequired[str]
    """
    Function name
    """

    description: typing_extensions.NotRequired[str]
    """
    Function description
    """

    parameters: typing_extensions.NotRequired[typing.Dict[str, typing.Any]]
    """
    Function parameters
    """

    endpoint: typing_extensions.NotRequired[ThinkSettingsV1FunctionsItemEndpointParams]
    """
    The Function endpoint to call. if not passed, function is called client-side
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ReadV1ResponseMetadataMetadataSummaryInfoParams(typing_extensions.TypedDict):
    model_uuid: typing_extensions.NotRequired[str]
    input_tokens: typing_extensions.NotRequired[float]
    output_tokens: typing_extensions.NotRequired[float]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .aws_polly_speak_provider_credentials import AwsPollySpeakProviderCredentials
from .aws_polly_speak_provider_engine import AwsPollySpeakProviderEngine
from .aws_polly_speak_provider_voice import AwsPollySpeakProviderVoice


class AwsPollySpeakProvider(UniversalBaseModel):
    type: typing.Literal["aws_polly"] = "aws_polly"
    voice: AwsPollySpeakProviderVoice = pydantic.Field()
    """
    AWS Polly voice name
    """

    language: str = pydantic.Field()
    """
    Language code to use, e.g. 'en-US'. Corresponds to the `language_code` parameter in the AWS Polly API
    """

    language_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    Use the `language` field instead.
    """

    engine: AwsPollySpeakProviderEngine
    credentials: AwsPollySpeakProviderCredentials

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .listen_v1response_metadata_intents_info import ListenV1ResponseMetadataIntentsInfo
from .listen_v1response_metadata_sentiment_info import ListenV1ResponseMetadataSentimentInfo
from .listen_v1response_metadata_summary_info import ListenV1ResponseMetadataSummaryInfo
from .listen_v1response_metadata_topics_info import ListenV1ResponseMetadataTopicsInfo


class ListenV1ResponseMetadata(UniversalBaseModel):
    transaction_key: typing.Optional[str] = None
    request_id: str
    sha256: str
    created: dt.datetime
    duration: float
    channels: float
    models: typing.List[str]
    model_info: typing.Dict[str, typing.Any]
    summary_info: typing.Optional[ListenV1ResponseMetadataSummaryInfo] = None
    sentiment_info: typing.Optional[ListenV1ResponseMetadataSentimentInfo] = None
    topics_info: typing.Optional[ListenV1ResponseMetadataTopicsInfo] = None
    intents_info: typing.Optional[ListenV1ResponseMetadataIntentsInfo] = None
    tags: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class AgentV1SettingsFlagsParams(typing_extensions.TypedDict):
    history: typing_extensions.NotRequired[bool]
    """
    Enable or disable history message reporting
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class AgentV1SettingsAgentContextMessagesItemFunctionCallsFunctionCallsItemParams(typing_extensions.TypedDict):
    id: str
    """
    Unique identifier for the function call
    """

    name: str
    """
    Name of the function called
    """

    client_side: bool
    """
    Indicates if the call was client-side or server-side
    """

    arguments: str
    """
    Arguments passed to the function
    """

    response: str
    """
    Response from the function call
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .listen_v1response_results_channels_item_alternatives_item_entities_item import (
    ListenV1ResponseResultsChannelsItemAlternativesItemEntitiesItem,
)
from .listen_v1response_results_channels_item_alternatives_item_paragraphs import (
    ListenV1ResponseResultsChannelsItemAlternativesItemParagraphs,
)
from .listen_v1response_results_channels_item_alternatives_item_summaries_item import (
    ListenV1ResponseResultsChannelsItemAlternativesItemSummariesItem,
)
from .listen_v1response_results_channels_item_alternatives_item_topics_item import (
    ListenV1ResponseResultsChannelsItemAlternativesItemTopicsItem,
)
from .listen_v1response_results_channels_item_alternatives_item_words_item import (
    ListenV1ResponseResultsChannelsItemAlternativesItemWordsItem,
)


class ListenV1ResponseResultsChannelsItemAlternativesItem(UniversalBaseModel):
    transcript: typing.Optional[str] = None
    confidence: typing.Optional[float] = None
    words: typing.Optional[typing.List[ListenV1ResponseResultsChannelsItemAlternativesItemWordsItem]] = None
    paragraphs: typing.Optional[ListenV1ResponseResultsChannelsItemAlternativesItemParagraphs] = None
    entities: typing.Optional[typing.List[ListenV1ResponseResultsChannelsItemAlternativesItemEntitiesItem]] = None
    summaries: typing.Optional[typing.List[ListenV1ResponseResultsChannelsItemAlternativesItemSummariesItem]] = None
    topics: typing.Optional[typing.List[ListenV1ResponseResultsChannelsItemAlternativesItemTopicsItem]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SpeakSettingsV1Endpoint(UniversalBaseModel):
    """
    Optional if provider is Deepgram. Required for non-Deepgram TTS providers.
    When present, must include url field and headers object. Valid schemes are https and wss with wss only supported for Eleven Labs.
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Custom TTS endpoint URL. Cannot contain `output_format` or `model_id` query parameters when the provider is Eleven Labs.
    """

    headers: typing.Optional[typing.Dict[str, str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ......core.request_options import RequestOptions
from ......types.billing_breakdown_v1response import BillingBreakdownV1Response
from .raw_client import AsyncRawBreakdownClient, RawBreakdownClient
from .types.breakdown_list_request_deployment import BreakdownListRequestDeployment
from .types.breakdown_list_request_grouping_item import BreakdownListRequestGroupingItem


class BreakdownClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBreakdownClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawBreakdownClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBreakdownClient
        """
        return self._raw_client

    def list(
        self,
        project_id: str,
        *,
        start: typing.Optional[str] = None,
        end: typing.Optional[str] = None,
        accessor: typing.Optional[str] = None,
        deployment: typing.Optional[BreakdownListRequestDeployment] = None,
        tag: typing.Optional[str] = None,
        line_item: typing.Optional[str] = None,
        grouping: typing.Optional[
            typing.Union[BreakdownListRequestGroupingItem, typing.Sequence[BreakdownListRequestGroupingItem]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BillingBreakdownV1Response:
        """
        Retrieves the billing summary for a specific project, with various filter options or by grouping options.

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        start : typing.Optional[str]
            Start date of the requested date range. Format accepted is YYYY-MM-DD

        end : typing.Optional[str]
            End date of the requested date range. Format accepted is YYYY-MM-DD

        accessor : typing.Optional[str]
            Filter for requests where a specific accessor was used

        deployment : typing.Optional[BreakdownListRequestDeployment]
            Filter for requests where a specific deployment was used

        tag : typing.Optional[str]
            Filter for requests where a specific tag was used

        line_item : typing.Optional[str]
            Filter requests by line item (e.g. streaming::nova-3)

        grouping : typing.Optional[typing.Union[BreakdownListRequestGroupingItem, typing.Sequence[BreakdownListRequestGroupingItem]]]
            Group billing breakdown by one or more dimensions (accessor, deployment, line_item, tags)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillingBreakdownV1Response
            Billing breakdown response

        Examples
        --------
        from deepgram import DeepgramClient

        client = DeepgramClient(
            api_key="YOUR_API_KEY",
        )
        client.manage.v1.projects.billing.breakdown.list(
            project_id="123456-7890-1234-5678-901234",
            start="start",
            end="end",
            accessor="12345678-1234-1234-1234-123456789012",
            deployment="hosted",
            tag="tag1",
            line_item="streaming::nova-3",
        )
        """
        _response = self._raw_client.list(
            project_id,
            start=start,
            end=end,
            accessor=accessor,
            deployment=deployment,
            tag=tag,
            line_item=line_item,
            grouping=grouping,
            request_options=request_options,
        )
        return _response.data


class AsyncBreakdownClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBreakdownClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawBreakdownClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBreakdownClient
        """
        return self._raw_client

    async def list(
        self,
        project_id: str,
        *,
        start: typing.Optional[str] = None,
        end: typing.Optional[str] = None,
        accessor: typing.Optional[str] = None,
        deployment: typing.Optional[BreakdownListRequestDeployment] = None,
        tag: typing.Optional[str] = None,
        line_item: typing.Optional[str] = None,
        grouping: typing.Optional[
            typing.Union[BreakdownListRequestGroupingItem, typing.Sequence[BreakdownListRequestGroupingItem]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BillingBreakdownV1Response:
        """
        Retrieves the billing summary for a specific project, with various filter options or by grouping options.

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        start : typing.Optional[str]
            Start date of the requested date range. Format accepted is YYYY-MM-DD

        end : typing.Optional[str]
            End date of the requested date range. Format accepted is YYYY-MM-DD

        accessor : typing.Optional[str]
            Filter for requests where a specific accessor was used

        deployment : typing.Optional[BreakdownListRequestDeployment]
            Filter for requests where a specific deployment was used

        tag : typing.Optional[str]
            Filter for requests where a specific tag was used

        line_item : typing.Optional[str]
            Filter requests by line item (e.g. streaming::nova-3)

        grouping : typing.Optional[typing.Union[BreakdownListRequestGroupingItem, typing.Sequence[BreakdownListRequestGroupingItem]]]
            Group billing breakdown by one or more dimensions (accessor, deployment, line_item, tags)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillingBreakdownV1Response
            Billing breakdown response

        Examples
        --------
        import asyncio

        from deepgram import AsyncDeepgramClient

        client = AsyncDeepgramClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.manage.v1.projects.billing.breakdown.list(
                project_id="123456-7890-1234-5678-901234",
                start="start",
                end="end",
                accessor="12345678-1234-1234-1234-123456789012",
                deployment="hosted",
                tag="tag1",
                line_item="streaming::nova-3",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            project_id,
            start=start,
            end=end,
            accessor=accessor,
            deployment=deployment,
            tag=tag,
            line_item=line_item,
            grouping=grouping,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.delete_project_v1response import DeleteProjectV1Response
from ....types.get_project_v1response import GetProjectV1Response
from ....types.leave_project_v1response import LeaveProjectV1Response
from ....types.list_projects_v1response import ListProjectsV1Response
from ....types.update_project_v1response import UpdateProjectV1Response
from .raw_client import AsyncRawProjectsClient, RawProjectsClient

if typing.TYPE_CHECKING:
    from .billing.client import AsyncBillingClient, BillingClient
    from .keys.client import AsyncKeysClient, KeysClient
    from .members.client import AsyncMembersClient, MembersClient
    from .models.client import AsyncModelsClient, ModelsClient
    from .requests.client import AsyncRequestsClient, RequestsClient
    from .usage.client import AsyncUsageClient, UsageClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ProjectsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawProjectsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._keys: typing.Optional[KeysClient] = None
        self._members: typing.Optional[MembersClient] = None
        self._models: typing.Optional[ModelsClient] = None
        self._requests: typing.Optional[RequestsClient] = None
        self._usage: typing.Optional[UsageClient] = None
        self._billing: typing.Optional[BillingClient] = None

    @property
    def with_raw_response(self) -> RawProjectsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawProjectsClient
        """
        return self._raw_client

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListProjectsV1Response:
        """
        Retrieves basic information about the projects associated with the API key

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListProjectsV1Response
            A list of projects

        Examples
        --------
        from deepgram import DeepgramClient

        client = DeepgramClient(
            api_key="YOUR_API_KEY",
        )
        client.manage.v1.projects.list()
        """
        _response = self._raw_client.list(request_options=request_options)
        return _response.data

    def get(
        self,
        project_id: str,
        *,
        limit: typing.Optional[float] = None,
        page: typing.Optional[float] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetProjectV1Response:
        """
        Retrieves information about the specified project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        limit : typing.Optional[float]
            Number of results to return per page. Default 10. Range [1,1000]

        page : typing.Optional[float]
            Navigate and return the results to retrieve specific portions of information of the response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetProjectV1Response
            A project

        Examples
        --------
        from deepgram import DeepgramClient

        client = DeepgramClient(
            api_key="YOUR_API_KEY",
        )
        client.manage.v1.projects.get(
            project_id="123456-7890-1234-5678-901234",
            limit=1.1,
            page=1.1,
        )
        """
        _response = self._raw_client.get(project_id, limit=limit, page=page, request_options=request_options)
        return _response.data

    def delete(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteProjectV1Response:
        """
        Deletes the specified project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteProjectV1Response
            A project

        Examples
        --------
        from deepgram import DeepgramClient

        client = DeepgramClient(
            api_key="YOUR_API_KEY",
        )
        client.manage.v1.projects.delete(
            project_id="123456-7890-1234-5678-901234",
        )
        """
        _response = self._raw_client.delete(project_id, request_options=request_options)
        return _response.data

    def update(
        self,
        project_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateProjectV1Response:
        """
        Updates the name or other properties of an existing project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        name : typing.Optional[str]
            The name of the project

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateProjectV1Response
            A project

        Examples
        --------
        from deepgram import DeepgramClient

        client = DeepgramClient(
            api_key="YOUR_API_KEY",
        )
        client.manage.v1.projects.update(
            project_id="123456-7890-1234-5678-901234",
        )
        """
        _response = self._raw_client.update(project_id, name=name, request_options=request_options)
        return _response.data

    def leave(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> LeaveProjectV1Response:
        """
        Removes the authenticated account from the specific project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LeaveProjectV1Response
            Successfully removed account from project

        Examples
        --------
        from deepgram import DeepgramClient

        client = DeepgramClient(
            api_key="YOUR_API_KEY",
        )
        client.manage.v1.projects.leave(
            project_id="123456-7890-1234-5678-901234",
        )
        """
        _response = self._raw_client.leave(project_id, request_options=request_options)
        return _response.data

    @property
    def keys(self):
        if self._keys is None:
            from .keys.client import KeysClient  # noqa: E402

            self._keys = KeysClient(client_wrapper=self._client_wrapper)
        return self._keys

    @property
    def members(self):
        if self._members is None:
            from .members.client import MembersClient  # noqa: E402

            self._members = MembersClient(client_wrapper=self._client_wrapper)
        return self._members

    @property
    def models(self):
        if self._models is None:
            from .models.client import ModelsClient  # noqa: E402

            self._models = ModelsClient(client_wrapper=self._client_wrapper)
        return self._models

    @property
    def requests(self):
        if self._requests is None:
            from .requests.client import RequestsClient  # noqa: E402

            self._requests = RequestsClient(client_wrapper=self._client_wrapper)
        return self._requests

    @property
    def usage(self):
        if self._usage is None:
            from .usage.client import UsageClient  # noqa: E402

            self._usage = UsageClient(client_wrapper=self._client_wrapper)
        return self._usage

    @property
    def billing(self):
        if self._billing is None:
            from .billing.client import BillingClient  # noqa: E402

            self._billing = BillingClient(client_wrapper=self._client_wrapper)
        return self._billing


class AsyncProjectsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawProjectsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._keys: typing.Optional[AsyncKeysClient] = None
        self._members: typing.Optional[AsyncMembersClient] = None
        self._models: typing.Optional[AsyncModelsClient] = None
        self._requests: typing.Optional[AsyncRequestsClient] = None
        self._usage: typing.Optional[AsyncUsageClient] = None
        self._billing: typing.Optional[AsyncBillingClient] = None

    @property
    def with_raw_response(self) -> AsyncRawProjectsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawProjectsClient
        """
        return self._raw_client

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListProjectsV1Response:
        """
        Retrieves basic information about the projects associated with the API key

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListProjectsV1Response
            A list of projects

        Examples
        --------
        import asyncio

        from deepgram import AsyncDeepgramClient

        client = AsyncDeepgramClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.manage.v1.projects.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(request_options=request_options)
        return _response.data

    async def get(
        self,
        project_id: str,
        *,
        limit: typing.Optional[float] = None,
        page: typing.Optional[float] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetProjectV1Response:
        """
        Retrieves information about the specified project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        limit : typing.Optional[float]
            Number of results to return per page. Default 10. Range [1,1000]

        page : typing.Optional[float]
            Navigate and return the results to retrieve specific portions of information of the response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetProjectV1Response
            A project

        Examples
        --------
        import asyncio

        from deepgram import AsyncDeepgramClient

        client = AsyncDeepgramClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.manage.v1.projects.get(
                project_id="123456-7890-1234-5678-901234",
                limit=1.1,
                page=1.1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(project_id, limit=limit, page=page, request_options=request_options)
        return _response.data

    async def delete(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteProjectV1Response:
        """
        Deletes the specified project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteProjectV1Response
            A project

        Examples
        --------
        import asyncio

        from deepgram import AsyncDeepgramClient

        client = AsyncDeepgramClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.manage.v1.projects.delete(
                project_id="123456-7890-1234-5678-901234",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(project_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        project_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateProjectV1Response:
        """
        Updates the name or other properties of an existing project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        name : typing.Optional[str]
            The name of the project

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateProjectV1Response
            A project

        Examples
        --------
        import asyncio

        from deepgram import AsyncDeepgramClient

        client = AsyncDeepgramClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.manage.v1.projects.update(
                project_id="123456-7890-1234-5678-901234",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(project_id, name=name, request_options=request_options)
        return _response.data

    async def leave(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> LeaveProjectV1Response:
        """
        Removes the authenticated account from the specific project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LeaveProjectV1Response
            Successfully removed account from project

        Examples
        --------
        import asyncio

        from deepgram import AsyncDeepgramClient

        client = AsyncDeepgramClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.manage.v1.projects.leave(
                project_id="123456-7890-1234-5678-901234",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.leave(project_id, request_options=request_options)
        return _response.data

    @property
    def keys(self):
        if self._keys is None:
            from .keys.client import AsyncKeysClient  # noqa: E402

            self._keys = AsyncKeysClient(client_wrapper=self._client_wrapper)
        return self._keys

    @property
    def members(self):
        if self._members is None:
            from .members.client import AsyncMembersClient  # noqa: E402

            self._members = AsyncMembersClient(client_wrapper=self._client_wrapper)
        return self._members

    @property
    def models(self):
        if self._models is None:
            from .models.client import AsyncModelsClient  # noqa: E402

            self._models = AsyncModelsClient(client_wrapper=self._client_wrapper)
        return self._models

    @property
    def requests(self):
        if self._requests is None:
            from .requests.client import AsyncRequestsClient  # noqa: E402

            self._requests = AsyncRequestsClient(client_wrapper=self._client_wrapper)
        return self._requests

    @property
    def usage(self):
        if self._usage is None:
            from .usage.client import AsyncUsageClient  # noqa: E402

            self._usage = AsyncUsageClient(client_wrapper=self._client_wrapper)
        return self._usage

    @property
    def billing(self):
        if self._billing is None:
            from .billing.client import AsyncBillingClient  # noqa: E402

            self._billing = AsyncBillingClient(client_wrapper=self._client_wrapper)
        return self._billing



# This file was auto-generated by Fern from our API Definition.

import typing

DistributionCredentialsCreateRequestScopesItem = typing.Union[
    typing.Literal[
        "self-hosted:products",
        "self-hosted:product:api",
        "self-hosted:product:engine",
        "self-hosted:product:license-proxy",
        "self-hosted:product:dgtools",
        "self-hosted:product:billing",
        "self-hosted:product:hotpepper",
        "self-hosted:product:metrics-server",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import typing_extensions
from .listen_v1response_metadata_intents_info import ListenV1ResponseMetadataIntentsInfoParams
from .listen_v1response_metadata_sentiment_info import ListenV1ResponseMetadataSentimentInfoParams
from .listen_v1response_metadata_summary_info import ListenV1ResponseMetadataSummaryInfoParams
from .listen_v1response_metadata_topics_info import ListenV1ResponseMetadataTopicsInfoParams


class ListenV1ResponseMetadataParams(typing_extensions.TypedDict):
    transaction_key: typing_extensions.NotRequired[str]
    request_id: str
    sha256: str
    created: dt.datetime
    duration: float
    channels: float
    models: typing.Sequence[str]
    model_info: typing.Dict[str, typing.Any]
    summary_info: typing_extensions.NotRequired[ListenV1ResponseMetadataSummaryInfoParams]
    sentiment_info: typing_extensions.NotRequired[ListenV1ResponseMetadataSentimentInfoParams]
    topics_info: typing_extensions.NotRequired[ListenV1ResponseMetadataTopicsInfoParams]
    intents_info: typing_extensions.NotRequired[ListenV1ResponseMetadataIntentsInfoParams]
    tags: typing_extensions.NotRequired[typing.Sequence[str]]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.speak_v1flushed_type import SpeakV1FlushedType


class SpeakV1FlushedParams(typing_extensions.TypedDict):
    type: SpeakV1FlushedType
    """
    Message type identifier
    """

    sequence_id: float
    """
    The sequence ID of the response
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from .list_project_purchases_v1response_orders_item import ListProjectPurchasesV1ResponseOrdersItemParams


class ListProjectPurchasesV1ResponseParams(typing_extensions.TypedDict):
    orders: typing_extensions.NotRequired[typing.Sequence[ListProjectPurchasesV1ResponseOrdersItemParams]]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import typing_extensions


class ProjectRequestResponseParams(typing_extensions.TypedDict):
    """
    A single request
    """

    request_id: typing_extensions.NotRequired[str]
    """
    The unique identifier of the request
    """

    project_uuid: typing_extensions.NotRequired[str]
    """
    The unique identifier of the project
    """

    created: typing_extensions.NotRequired[dt.datetime]
    """
    The date and time the request was created
    """

    path: typing_extensions.NotRequired[str]
    """
    The API path of the request
    """

    api_key_id: typing_extensions.NotRequired[str]
    """
    The unique identifier of the API key
    """

    response: typing_extensions.NotRequired[typing.Dict[str, typing.Any]]
    """
    The response of the request
    """

    code: typing_extensions.NotRequired[float]
    """
    The response code of the request
    """

    deployment: typing_extensions.NotRequired[str]
    """
    The deployment type
    """

    callback: typing_extensions.NotRequired[str]
    """
    The callback URL for the request
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .agent_v1settings_audio_output_encoding import AgentV1SettingsAudioOutputEncoding


class AgentV1SettingsAudioOutput(UniversalBaseModel):
    """
    Audio output configuration settings
    """

    encoding: typing.Optional[AgentV1SettingsAudioOutputEncoding] = pydantic.Field(default=None)
    """
    Audio encoding format for streaming TTS output
    """

    sample_rate: typing.Optional[float] = pydantic.Field(default=None)
    """
    Sample rate in Hz
    """

    bitrate: typing.Optional[float] = pydantic.Field(default=None)
    """
    Audio bitrate in bits per second
    """

    container: typing.Optional[str] = pydantic.Field(default=None)
    """
    Audio container format. If omitted, defaults to 'none'
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.listen_v1keep_alive_type import ListenV1KeepAliveType


class ListenV1KeepAliveParams(typing_extensions.TypedDict):
    type: ListenV1KeepAliveType
    """
    Message type identifier
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .billing_breakdown_v1response_results_item_grouping import BillingBreakdownV1ResponseResultsItemGroupingParams


class BillingBreakdownV1ResponseResultsItemParams(typing_extensions.TypedDict):
    dollars: float
    """
    USD cost of the billing for this grouping
    """

    grouping: BillingBreakdownV1ResponseResultsItemGroupingParams



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ......core.request_options import RequestOptions
from ......types.usage_breakdown_v1response import UsageBreakdownV1Response
from .raw_client import AsyncRawBreakdownClient, RawBreakdownClient
from .types.breakdown_get_request_deployment import BreakdownGetRequestDeployment
from .types.breakdown_get_request_endpoint import BreakdownGetRequestEndpoint
from .types.breakdown_get_request_grouping import BreakdownGetRequestGrouping
from .types.breakdown_get_request_method import BreakdownGetRequestMethod


class BreakdownClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBreakdownClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawBreakdownClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBreakdownClient
        """
        return self._raw_client

    def get(
        self,
        project_id: str,
        *,
        start: typing.Optional[str] = None,
        end: typing.Optional[str] = None,
        grouping: typing.Optional[BreakdownGetRequestGrouping] = None,
        accessor: typing.Optional[str] = None,
        alternatives: typing.Optional[bool] = None,
        callback_method: typing.Optional[bool] = None,
        callback: typing.Optional[bool] = None,
        channels: typing.Optional[bool] = None,
        custom_intent_mode: typing.Optional[bool] = None,
        custom_intent: typing.Optional[bool] = None,
        custom_topic_mode: typing.Optional[bool] = None,
        custom_topic: typing.Optional[bool] = None,
        deployment: typing.Optional[BreakdownGetRequestDeployment] = None,
        detect_entities: typing.Optional[bool] = None,
        detect_language: typing.Optional[bool] = None,
        diarize: typing.Optional[bool] = None,
        dictation: typing.Optional[bool] = None,
        encoding: typing.Optional[bool] = None,
        endpoint: typing.Optional[BreakdownGetRequestEndpoint] = None,
        extra: typing.Optional[bool] = None,
        filler_words: typing.Optional[bool] = None,
        intents: typing.Optional[bool] = None,
        keyterm: typing.Optional[bool] = None,
        keywords: typing.Optional[bool] = None,
        language: typing.Optional[bool] = None,
        measurements: typing.Optional[bool] = None,
        method: typing.Optional[BreakdownGetRequestMethod] = None,
        model: typing.Optional[str] = None,
        multichannel: typing.Optional[bool] = None,
        numerals: typing.Optional[bool] = None,
        paragraphs: typing.Optional[bool] = None,
        profanity_filter: typing.Optional[bool] = None,
        punctuate: typing.Optional[bool] = None,
        redact: typing.Optional[bool] = None,
        replace: typing.Optional[bool] = None,
        sample_rate: typing.Optional[bool] = None,
        search: typing.Optional[bool] = None,
        sentiment: typing.Optional[bool] = None,
        smart_format: typing.Optional[bool] = None,
        summarize: typing.Optional[bool] = None,
        tag: typing.Optional[str] = None,
        topics: typing.Optional[bool] = None,
        utt_split: typing.Optional[bool] = None,
        utterances: typing.Optional[bool] = None,
        version: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UsageBreakdownV1Response:
        """
        Retrieves the usage breakdown for a specific project, with various filter options by API feature or by groupings. Setting a feature (e.g. diarize) to true includes requests that used that feature, while false excludes requests that used it. Multiple true filters are combined with OR logic, while false filters use AND logic.

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        start : typing.Optional[str]
            Start date of the requested date range. Format accepted is YYYY-MM-DD

        end : typing.Optional[str]
            End date of the requested date range. Format accepted is YYYY-MM-DD

        grouping : typing.Optional[BreakdownGetRequestGrouping]
            Common usage grouping parameters

        accessor : typing.Optional[str]
            Filter for requests where a specific accessor was used

        alternatives : typing.Optional[bool]
            Filter for requests where alternatives were used

        callback_method : typing.Optional[bool]
            Filter for requests where callback method was used

        callback : typing.Optional[bool]
            Filter for requests where callback was used

        channels : typing.Optional[bool]
            Filter for requests where channels were used

        custom_intent_mode : typing.Optional[bool]
            Filter for requests where custom intent mode was used

        custom_intent : typing.Optional[bool]
            Filter for requests where custom intent was used

        custom_topic_mode : typing.Optional[bool]
            Filter for requests where custom topic mode was used

        custom_topic : typing.Optional[bool]
            Filter for requests where custom topic was used

        deployment : typing.Optional[BreakdownGetRequestDeployment]
            Filter for requests where a specific deployment was used

        detect_entities : typing.Optional[bool]
            Filter for requests where detect entities was used

        detect_language : typing.Optional[bool]
            Filter for requests where detect language was used

        diarize : typing.Optional[bool]
            Filter for requests where diarize was used

        dictation : typing.Optional[bool]
            Filter for requests where dictation was used

        encoding : typing.Optional[bool]
            Filter for requests where encoding was used

        endpoint : typing.Optional[BreakdownGetRequestEndpoint]
            Filter for requests where a specific endpoint was used

        extra : typing.Optional[bool]
            Filter for requests where extra was used

        filler_words : typing.Optional[bool]
            Filter for requests where filler words was used

        intents : typing.Optional[bool]
            Filter for requests where intents was used

        keyterm : typing.Optional[bool]
            Filter for requests where keyterm was used

        keywords : typing.Optional[bool]
            Filter for requests where keywords was used

        language : typing.Optional[bool]
            Filter for requests where language was used

        measurements : typing.Optional[bool]
            Filter for requests where measurements were used

        method : typing.Optional[BreakdownGetRequestMethod]
            Filter for requests where a specific method was used

        model : typing.Optional[str]
            Filter for requests where a specific model uuid was used

        multichannel : typing.Optional[bool]
            Filter for requests where multichannel was used

        numerals : typing.Optional[bool]
            Filter for requests where numerals were used

        paragraphs : typing.Optional[bool]
            Filter for requests where paragraphs were used

        profanity_filter : typing.Optional[bool]
            Filter for requests where profanity filter was used

        punctuate : typing.Optional[bool]
            Filter for requests where punctuate was used

        redact : typing.Optional[bool]
            Filter for requests where redact was used

        replace : typing.Optional[bool]
            Filter for requests where replace was used

        sample_rate : typing.Optional[bool]
            Filter for requests where sample rate was used

        search : typing.Optional[bool]
            Filter for requests where search was used

        sentiment : typing.Optional[bool]
            Filter for requests where sentiment was used

        smart_format : typing.Optional[bool]
            Filter for requests where smart format was used

        summarize : typing.Optional[bool]
            Filter for requests where summarize was used

        tag : typing.Optional[str]
            Filter for requests where a specific tag was used

        topics : typing.Optional[bool]
            Filter for requests where topics was used

        utt_split : typing.Optional[bool]
            Filter for requests where utt split was used

        utterances : typing.Optional[bool]
            Filter for requests where utterances was used

        version : typing.Optional[bool]
            Filter for requests where version was used

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UsageBreakdownV1Response
            Usage breakdown response

        Examples
        --------
        from deepgram import DeepgramClient

        client = DeepgramClient(
            api_key="YOUR_API_KEY",
        )
        client.manage.v1.projects.usage.breakdown.get(
            project_id="123456-7890-1234-5678-901234",
            start="start",
            end="end",
            grouping="accessor",
            accessor="12345678-1234-1234-1234-123456789012",
            alternatives=True,
            callback_method=True,
            callback=True,
            channels=True,
            custom_intent_mode=True,
            custom_intent=True,
            custom_topic_mode=True,
            custom_topic=True,
            deployment="hosted",
            detect_entities=True,
            detect_language=True,
            diarize=True,
            dictation=True,
            encoding=True,
            endpoint="listen",
            extra=True,
            filler_words=True,
            intents=True,
            keyterm=True,
            keywords=True,
            language=True,
            measurements=True,
            method="sync",
            model="6f548761-c9c0-429a-9315-11a1d28499c8",
            multichannel=True,
            numerals=True,
            paragraphs=True,
            profanity_filter=True,
            punctuate=True,
            redact=True,
            replace=True,
            sample_rate=True,
            search=True,
            sentiment=True,
            smart_format=True,
            summarize=True,
            tag="tag1",
            topics=True,
            utt_split=True,
            utterances=True,
            version=True,
        )
        """
        _response = self._raw_client.get(
            project_id,
            start=start,
            end=end,
            grouping=grouping,
            accessor=accessor,
            alternatives=alternatives,
            callback_method=callback_method,
            callback=callback,
            channels=channels,
            custom_intent_mode=custom_intent_mode,
            custom_intent=custom_intent,
            custom_topic_mode=custom_topic_mode,
            custom_topic=custom_topic,
            deployment=deployment,
            detect_entities=detect_entities,
            detect_language=detect_language,
            diarize=diarize,
            dictation=dictation,
            encoding=encoding,
            endpoint=endpoint,
            extra=extra,
            filler_words=filler_words,
            intents=intents,
            keyterm=keyterm,
            keywords=keywords,
            language=language,
            measurements=measurements,
            method=method,
            model=model,
            multichannel=multichannel,
            numerals=numerals,
            paragraphs=paragraphs,
            profanity_filter=profanity_filter,
            punctuate=punctuate,
            redact=redact,
            replace=replace,
            sample_rate=sample_rate,
            search=search,
            sentiment=sentiment,
            smart_format=smart_format,
            summarize=summarize,
            tag=tag,
            topics=topics,
            utt_split=utt_split,
            utterances=utterances,
            version=version,
            request_options=request_options,
        )
        return _response.data


class AsyncBreakdownClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBreakdownClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawBreakdownClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBreakdownClient
        """
        return self._raw_client

    async def get(
        self,
        project_id: str,
        *,
        start: typing.Optional[str] = None,
        end: typing.Optional[str] = None,
        grouping: typing.Optional[BreakdownGetRequestGrouping] = None,
        accessor: typing.Optional[str] = None,
        alternatives: typing.Optional[bool] = None,
        callback_method: typing.Optional[bool] = None,
        callback: typing.Optional[bool] = None,
        channels: typing.Optional[bool] = None,
        custom_intent_mode: typing.Optional[bool] = None,
        custom_intent: typing.Optional[bool] = None,
        custom_topic_mode: typing.Optional[bool] = None,
        custom_topic: typing.Optional[bool] = None,
        deployment: typing.Optional[BreakdownGetRequestDeployment] = None,
        detect_entities: typing.Optional[bool] = None,
        detect_language: typing.Optional[bool] = None,
        diarize: typing.Optional[bool] = None,
        dictation: typing.Optional[bool] = None,
        encoding: typing.Optional[bool] = None,
        endpoint: typing.Optional[BreakdownGetRequestEndpoint] = None,
        extra: typing.Optional[bool] = None,
        filler_words: typing.Optional[bool] = None,
        intents: typing.Optional[bool] = None,
        keyterm: typing.Optional[bool] = None,
        keywords: typing.Optional[bool] = None,
        language: typing.Optional[bool] = None,
        measurements: typing.Optional[bool] = None,
        method: typing.Optional[BreakdownGetRequestMethod] = None,
        model: typing.Optional[str] = None,
        multichannel: typing.Optional[bool] = None,
        numerals: typing.Optional[bool] = None,
        paragraphs: typing.Optional[bool] = None,
        profanity_filter: typing.Optional[bool] = None,
        punctuate: typing.Optional[bool] = None,
        redact: typing.Optional[bool] = None,
        replace: typing.Optional[bool] = None,
        sample_rate: typing.Optional[bool] = None,
        search: typing.Optional[bool] = None,
        sentiment: typing.Optional[bool] = None,
        smart_format: typing.Optional[bool] = None,
        summarize: typing.Optional[bool] = None,
        tag: typing.Optional[str] = None,
        topics: typing.Optional[bool] = None,
        utt_split: typing.Optional[bool] = None,
        utterances: typing.Optional[bool] = None,
        version: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UsageBreakdownV1Response:
        """
        Retrieves the usage breakdown for a specific project, with various filter options by API feature or by groupings. Setting a feature (e.g. diarize) to true includes requests that used that feature, while false excludes requests that used it. Multiple true filters are combined with OR logic, while false filters use AND logic.

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        start : typing.Optional[str]
            Start date of the requested date range. Format accepted is YYYY-MM-DD

        end : typing.Optional[str]
            End date of the requested date range. Format accepted is YYYY-MM-DD

        grouping : typing.Optional[BreakdownGetRequestGrouping]
            Common usage grouping parameters

        accessor : typing.Optional[str]
            Filter for requests where a specific accessor was used

        alternatives : typing.Optional[bool]
            Filter for requests where alternatives were used

        callback_method : typing.Optional[bool]
            Filter for requests where callback method was used

        callback : typing.Optional[bool]
            Filter for requests where callback was used

        channels : typing.Optional[bool]
            Filter for requests where channels were used

        custom_intent_mode : typing.Optional[bool]
            Filter for requests where custom intent mode was used

        custom_intent : typing.Optional[bool]
            Filter for requests where custom intent was used

        custom_topic_mode : typing.Optional[bool]
            Filter for requests where custom topic mode was used

        custom_topic : typing.Optional[bool]
            Filter for requests where custom topic was used

        deployment : typing.Optional[BreakdownGetRequestDeployment]
            Filter for requests where a specific deployment was used

        detect_entities : typing.Optional[bool]
            Filter for requests where detect entities was used

        detect_language : typing.Optional[bool]
            Filter for requests where detect language was used

        diarize : typing.Optional[bool]
            Filter for requests where diarize was used

        dictation : typing.Optional[bool]
            Filter for requests where dictation was used

        encoding : typing.Optional[bool]
            Filter for requests where encoding was used

        endpoint : typing.Optional[BreakdownGetRequestEndpoint]
            Filter for requests where a specific endpoint was used

        extra : typing.Optional[bool]
            Filter for requests where extra was used

        filler_words : typing.Optional[bool]
            Filter for requests where filler words was used

        intents : typing.Optional[bool]
            Filter for requests where intents was used

        keyterm : typing.Optional[bool]
            Filter for requests where keyterm was used

        keywords : typing.Optional[bool]
            Filter for requests where keywords was used

        language : typing.Optional[bool]
            Filter for requests where language was used

        measurements : typing.Optional[bool]
            Filter for requests where measurements were used

        method : typing.Optional[BreakdownGetRequestMethod]
            Filter for requests where a specific method was used

        model : typing.Optional[str]
            Filter for requests where a specific model uuid was used

        multichannel : typing.Optional[bool]
            Filter for requests where multichannel was used

        numerals : typing.Optional[bool]
            Filter for requests where numerals were used

        paragraphs : typing.Optional[bool]
            Filter for requests where paragraphs were used

        profanity_filter : typing.Optional[bool]
            Filter for requests where profanity filter was used

        punctuate : typing.Optional[bool]
            Filter for requests where punctuate was used

        redact : typing.Optional[bool]
            Filter for requests where redact was used

        replace : typing.Optional[bool]
            Filter for requests where replace was used

        sample_rate : typing.Optional[bool]
            Filter for requests where sample rate was used

        search : typing.Optional[bool]
            Filter for requests where search was used

        sentiment : typing.Optional[bool]
            Filter for requests where sentiment was used

        smart_format : typing.Optional[bool]
            Filter for requests where smart format was used

        summarize : typing.Optional[bool]
            Filter for requests where summarize was used

        tag : typing.Optional[str]
            Filter for requests where a specific tag was used

        topics : typing.Optional[bool]
            Filter for requests where topics was used

        utt_split : typing.Optional[bool]
            Filter for requests where utt split was used

        utterances : typing.Optional[bool]
            Filter for requests where utterances was used

        version : typing.Optional[bool]
            Filter for requests where version was used

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UsageBreakdownV1Response
            Usage breakdown response

        Examples
        --------
        import asyncio

        from deepgram import AsyncDeepgramClient

        client = AsyncDeepgramClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.manage.v1.projects.usage.breakdown.get(
                project_id="123456-7890-1234-5678-901234",
                start="start",
                end="end",
                grouping="accessor",
                accessor="12345678-1234-1234-1234-123456789012",
                alternatives=True,
                callback_method=True,
                callback=True,
                channels=True,
                custom_intent_mode=True,
                custom_intent=True,
                custom_topic_mode=True,
                custom_topic=True,
                deployment="hosted",
                detect_entities=True,
                detect_language=True,
                diarize=True,
                dictation=True,
                encoding=True,
                endpoint="listen",
                extra=True,
                filler_words=True,
                intents=True,
                keyterm=True,
                keywords=True,
                language=True,
                measurements=True,
                method="sync",
                model="6f548761-c9c0-429a-9315-11a1d28499c8",
                multichannel=True,
                numerals=True,
                paragraphs=True,
                profanity_filter=True,
                punctuate=True,
                redact=True,
                replace=True,
                sample_rate=True,
                search=True,
                sentiment=True,
                smart_format=True,
                summarize=True,
                tag="tag1",
                topics=True,
                utt_split=True,
                utterances=True,
                version=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(
            project_id,
            start=start,
            end=end,
            grouping=grouping,
            accessor=accessor,
            alternatives=alternatives,
            callback_method=callback_method,
            callback=callback,
            channels=channels,
            custom_intent_mode=custom_intent_mode,
            custom_intent=custom_intent,
            custom_topic_mode=custom_topic_mode,
            custom_topic=custom_topic,
            deployment=deployment,
            detect_entities=detect_entities,
            detect_language=detect_language,
            diarize=diarize,
            dictation=dictation,
            encoding=encoding,
            endpoint=endpoint,
            extra=extra,
            filler_words=filler_words,
            intents=intents,
            keyterm=keyterm,
            keywords=keywords,
            language=language,
            measurements=measurements,
            method=method,
            model=model,
            multichannel=multichannel,
            numerals=numerals,
            paragraphs=paragraphs,
            profanity_filter=profanity_filter,
            punctuate=punctuate,
            redact=redact,
            replace=replace,
            sample_rate=sample_rate,
            search=search,
            sentiment=sentiment,
            smart_format=smart_format,
            summarize=summarize,
            tag=tag,
            topics=topics,
            utt_split=utt_split,
            utterances=utterances,
            version=version,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .read_v1response_results_summary import ReadV1ResponseResultsSummaryParams
from .shared_intents import SharedIntentsParams
from .shared_sentiments import SharedSentimentsParams
from .shared_topics import SharedTopicsParams


class ReadV1ResponseResultsParams(typing_extensions.TypedDict):
    summary: typing_extensions.NotRequired[ReadV1ResponseResultsSummaryParams]
    topics: typing_extensions.NotRequired[SharedTopicsParams]
    intents: typing_extensions.NotRequired[SharedIntentsParams]
    sentiments: typing_extensions.NotRequired[SharedSentimentsParams]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class UsageV1ResponseResolutionParams(typing_extensions.TypedDict):
    units: typing_extensions.NotRequired[str]
    amount: typing_extensions.NotRequired[float]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ..types.listen_v1finalize_type import ListenV1FinalizeType


class ListenV1FinalizeParams(typing_extensions.TypedDict):
    type: ListenV1FinalizeType
    """
    Message type identifier
    """



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import typing_extensions


class ListProjectDistributionCredentialsV1ResponseDistributionCredentialsItemDistributionCredentialsParams(
    typing_extensions.TypedDict
):
    distribution_credentials_id: str
    """
    Unique identifier for the distribution credentials
    """

    provider: str
    """
    The provider of the distribution service
    """

    comment: typing_extensions.NotRequired[str]
    """
    Optional comment about the credentials
    """

    scopes: typing.Sequence[str]
    """
    List of permission scopes for the credentials
    """

    created: dt.datetime
    """
    Timestamp when the credentials were created
    """



# This file was auto-generated by Fern from our API Definition.

import typing

from .read_v1request_text import ReadV1RequestText
from .read_v1request_url import ReadV1RequestUrl

ReadV1Request = typing.Union[ReadV1RequestUrl, ReadV1RequestText]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class UsageBreakdownV1ResponseResultsItemGroupingParams(typing_extensions.TypedDict):
    start: typing_extensions.NotRequired[str]
    """
    Start date for this group
    """

    end: typing_extensions.NotRequired[str]
    """
    End date for this group
    """

    accessor: typing_extensions.NotRequired[str]
    """
    Optional accessor identifier
    """

    endpoint: typing_extensions.NotRequired[str]
    """
    Optional endpoint identifier
    """

    feature_set: typing_extensions.NotRequired[str]
    """
    Optional feature set identifier
    """

    models: typing_extensions.NotRequired[str]
    """
    Optional models identifier
    """

    method: typing_extensions.NotRequired[str]
    """
    Optional method identifier
    """

    tags: typing_extensions.NotRequired[str]
    """
    Optional tags
    """

    deployment: typing_extensions.NotRequired[str]
    """
    Optional deployment identifier
    """



# This file was auto-generated by Fern from our API Definition.

import typing

from ..types.error_response_text_error import ErrorResponseTextError
from .error_response_legacy_error import ErrorResponseLegacyErrorParams
from .error_response_modern_error import ErrorResponseModernErrorParams

ErrorResponseParams = typing.Union[
    ErrorResponseTextError, ErrorResponseLegacyErrorParams, ErrorResponseModernErrorParams
]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..types.list_billing_fields_v1response_deployments_item import ListBillingFieldsV1ResponseDeploymentsItem


class ListBillingFieldsV1ResponseParams(typing_extensions.TypedDict):
    accessors: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    List of accessor UUIDs for the time period
    """

    deployments: typing_extensions.NotRequired[typing.Sequence[ListBillingFieldsV1ResponseDeploymentsItem]]
    """
    List of deployment types for the time period
    """

    tags: typing_extensions.NotRequired[typing.Sequence[str]]
    """
    List of tags for the time period
    """

    line_items: typing_extensions.NotRequired[typing.Dict[str, str]]
    """
    Map of line item names to human-readable descriptions for the time period
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class AgentThinkModelsV1ResponseModelsItemThreeParams(typing_extensions.TypedDict):
    """
    Groq models
    """

    id: typing.Literal["openai/gpt-oss-20b"]
    """
    The unique identifier of the Groq model
    """

    name: str
    """
    The display name of the model
    """

    provider: typing.Literal["groq"]
    """
    The provider of the model
    """



# This file was auto-generated by Fern from our API Definition.

import typing

ListBillingFieldsV1ResponseDeploymentsItem = typing.Union[
    typing.Literal["hosted", "beta", "self-hosted", "dedicated"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class AgentV1WelcomeParams(typing_extensions.TypedDict):
    type: typing.Literal["Welcome"]
    """
    Message type identifier for welcome message
    """

    request_id: str
    """
    Unique identifier for the request
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .agent_v1settings_audio_input_encoding import AgentV1SettingsAudioInputEncoding


class AgentV1SettingsAudioInput(UniversalBaseModel):
    """
    Audio input configuration settings. If omitted, defaults to encoding=linear16 and sample_rate=24000. Higher sample rates like 44100 Hz provide better audio quality.
    """

    encoding: AgentV1SettingsAudioInputEncoding = pydantic.Field()
    """
    Audio encoding format
    """

    sample_rate: float = pydantic.Field()
    """
    Sample rate in Hz. Common values are 16000, 24000, 44100, 48000
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

AgentThinkModelsV1ResponseModelsItemTwoId = typing.Union[
    typing.Literal["gemini-2.5-flash", "gemini-2.0-flash", "gemini-2.0-flash-lite"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ....requests.speak_settings_v1 import SpeakSettingsV1Params

AgentV1SettingsAgentSpeakParams = typing.Union[SpeakSettingsV1Params, typing.Sequence[SpeakSettingsV1Params]]



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ......core.request_options import RequestOptions
from ......types.list_billing_fields_v1response import ListBillingFieldsV1Response
from .raw_client import AsyncRawFieldsClient, RawFieldsClient


class FieldsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawFieldsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawFieldsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawFieldsClient
        """
        return self._raw_client

    def list(
        self,
        project_id: str,
        *,
        start: typing.Optional[str] = None,
        end: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListBillingFieldsV1Response:
        """
        Lists the accessors, deployment types, tags, and line items used for billing data in the specified time period. Use this endpoint if you want to filter your results from the Billing Breakdown endpoint and want to know what filters are available.

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        start : typing.Optional[str]
            Start date of the requested date range. Format accepted is YYYY-MM-DD

        end : typing.Optional[str]
            End date of the requested date range. Format accepted is YYYY-MM-DD

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListBillingFieldsV1Response
            A list of billing fields for a specific project

        Examples
        --------
        from deepgram import DeepgramClient

        client = DeepgramClient(
            api_key="YOUR_API_KEY",
        )
        client.manage.v1.projects.billing.fields.list(
            project_id="123456-7890-1234-5678-901234",
            start="start",
            end="end",
        )
        """
        _response = self._raw_client.list(project_id, start=start, end=end, request_options=request_options)
        return _response.data


class AsyncFieldsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawFieldsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawFieldsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawFieldsClient
        """
        return self._raw_client

    async def list(
        self,
        project_id: str,
        *,
        start: typing.Optional[str] = None,
        end: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListBillingFieldsV1Response:
        """
        Lists the accessors, deployment types, tags, and line items used for billing data in the specified time period. Use this endpoint if you want to filter your results from the Billing Breakdown endpoint and want to know what filters are available.

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        start : typing.Optional[str]
            Start date of the requested date range. Format accepted is YYYY-MM-DD

        end : typing.Optional[str]
            End date of the requested date range. Format accepted is YYYY-MM-DD

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListBillingFieldsV1Response
            A list of billing fields for a specific project

        Examples
        --------
        import asyncio

        from deepgram import AsyncDeepgramClient

        client = AsyncDeepgramClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.manage.v1.projects.billing.fields.list(
                project_id="123456-7890-1234-5678-901234",
                start="start",
                end="end",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(project_id, start=start, end=end, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class AgentV1UserStartedSpeakingParams(typing_extensions.TypedDict):
    type: typing.Literal["UserStartedSpeaking"]
    """
    Message type identifier indicating that the user has begun speaking
    """



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.create_project_distribution_credentials_v1response import CreateProjectDistributionCredentialsV1Response
from ....types.get_project_distribution_credentials_v1response import GetProjectDistributionCredentialsV1Response
from ....types.list_project_distribution_credentials_v1response import ListProjectDistributionCredentialsV1Response
from .raw_client import AsyncRawDistributionCredentialsClient, RawDistributionCredentialsClient
from .types.distribution_credentials_create_request_scopes_item import DistributionCredentialsCreateRequestScopesItem

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DistributionCredentialsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDistributionCredentialsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDistributionCredentialsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDistributionCredentialsClient
        """
        return self._raw_client

    def list(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ListProjectDistributionCredentialsV1Response:
        """
        Lists sets of distribution credentials for the specified project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListProjectDistributionCredentialsV1Response
            A list of distribution credentials for a specific project

        Examples
        --------
        from deepgram import DeepgramClient

        client = DeepgramClient(
            api_key="YOUR_API_KEY",
        )
        client.self_hosted.v1.distribution_credentials.list(
            project_id="123456-7890-1234-5678-901234",
        )
        """
        _response = self._raw_client.list(project_id, request_options=request_options)
        return _response.data

    def create(
        self,
        project_id: str,
        *,
        scopes: typing.Optional[
            typing.Union[
                DistributionCredentialsCreateRequestScopesItem,
                typing.Sequence[DistributionCredentialsCreateRequestScopesItem],
            ]
        ] = None,
        provider: typing.Optional[typing.Literal["quay"]] = None,
        comment: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateProjectDistributionCredentialsV1Response:
        """
        Creates a set of distribution credentials for the specified project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        scopes : typing.Optional[typing.Union[DistributionCredentialsCreateRequestScopesItem, typing.Sequence[DistributionCredentialsCreateRequestScopesItem]]]
            List of permission scopes for the credentials

        provider : typing.Optional[typing.Literal["quay"]]
            The provider of the distribution service

        comment : typing.Optional[str]
            Optional comment about the credentials

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateProjectDistributionCredentialsV1Response
            Single distribution credential

        Examples
        --------
        from deepgram import DeepgramClient

        client = DeepgramClient(
            api_key="YOUR_API_KEY",
        )
        client.self_hosted.v1.distribution_credentials.create(
            project_id="123456-7890-1234-5678-901234",
        )
        """
        _response = self._raw_client.create(
            project_id, scopes=scopes, provider=provider, comment=comment, request_options=request_options
        )
        return _response.data

    def get(
        self,
        project_id: str,
        distribution_credentials_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetProjectDistributionCredentialsV1Response:
        """
        Returns a set of distribution credentials for the specified project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        distribution_credentials_id : str
            The UUID of the distribution credentials

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetProjectDistributionCredentialsV1Response
            Single distribution credential

        Examples
        --------
        from deepgram import DeepgramClient

        client = DeepgramClient(
            api_key="YOUR_API_KEY",
        )
        client.self_hosted.v1.distribution_credentials.get(
            project_id="123456-7890-1234-5678-901234",
            distribution_credentials_id="8b36cfd0-472f-4a21-833f-2d6343c3a2f3",
        )
        """
        _response = self._raw_client.get(project_id, distribution_credentials_id, request_options=request_options)
        return _response.data

    def delete(
        self,
        project_id: str,
        distribution_credentials_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetProjectDistributionCredentialsV1Response:
        """
        Deletes a set of distribution credentials for the specified project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        distribution_credentials_id : str
            The UUID of the distribution credentials

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetProjectDistributionCredentialsV1Response
            Single distribution credential

        Examples
        --------
        from deepgram import DeepgramClient

        client = DeepgramClient(
            api_key="YOUR_API_KEY",
        )
        client.self_hosted.v1.distribution_credentials.delete(
            project_id="123456-7890-1234-5678-901234",
            distribution_credentials_id="8b36cfd0-472f-4a21-833f-2d6343c3a2f3",
        )
        """
        _response = self._raw_client.delete(project_id, distribution_credentials_id, request_options=request_options)
        return _response.data


class AsyncDistributionCredentialsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDistributionCredentialsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDistributionCredentialsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDistributionCredentialsClient
        """
        return self._raw_client

    async def list(
        self, project_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ListProjectDistributionCredentialsV1Response:
        """
        Lists sets of distribution credentials for the specified project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListProjectDistributionCredentialsV1Response
            A list of distribution credentials for a specific project

        Examples
        --------
        import asyncio

        from deepgram import AsyncDeepgramClient

        client = AsyncDeepgramClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.self_hosted.v1.distribution_credentials.list(
                project_id="123456-7890-1234-5678-901234",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(project_id, request_options=request_options)
        return _response.data

    async def create(
        self,
        project_id: str,
        *,
        scopes: typing.Optional[
            typing.Union[
                DistributionCredentialsCreateRequestScopesItem,
                typing.Sequence[DistributionCredentialsCreateRequestScopesItem],
            ]
        ] = None,
        provider: typing.Optional[typing.Literal["quay"]] = None,
        comment: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateProjectDistributionCredentialsV1Response:
        """
        Creates a set of distribution credentials for the specified project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        scopes : typing.Optional[typing.Union[DistributionCredentialsCreateRequestScopesItem, typing.Sequence[DistributionCredentialsCreateRequestScopesItem]]]
            List of permission scopes for the credentials

        provider : typing.Optional[typing.Literal["quay"]]
            The provider of the distribution service

        comment : typing.Optional[str]
            Optional comment about the credentials

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateProjectDistributionCredentialsV1Response
            Single distribution credential

        Examples
        --------
        import asyncio

        from deepgram import AsyncDeepgramClient

        client = AsyncDeepgramClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.self_hosted.v1.distribution_credentials.create(
                project_id="123456-7890-1234-5678-901234",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            project_id, scopes=scopes, provider=provider, comment=comment, request_options=request_options
        )
        return _response.data

    async def get(
        self,
        project_id: str,
        distribution_credentials_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetProjectDistributionCredentialsV1Response:
        """
        Returns a set of distribution credentials for the specified project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        distribution_credentials_id : str
            The UUID of the distribution credentials

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetProjectDistributionCredentialsV1Response
            Single distribution credential

        Examples
        --------
        import asyncio

        from deepgram import AsyncDeepgramClient

        client = AsyncDeepgramClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.self_hosted.v1.distribution_credentials.get(
                project_id="123456-7890-1234-5678-901234",
                distribution_credentials_id="8b36cfd0-472f-4a21-833f-2d6343c3a2f3",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(project_id, distribution_credentials_id, request_options=request_options)
        return _response.data

    async def delete(
        self,
        project_id: str,
        distribution_credentials_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetProjectDistributionCredentialsV1Response:
        """
        Deletes a set of distribution credentials for the specified project

        Parameters
        ----------
        project_id : str
            The unique identifier of the project

        distribution_credentials_id : str
            The UUID of the distribution credentials

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetProjectDistributionCredentialsV1Response
            Single distribution credential

        Examples
        --------
        import asyncio

        from deepgram import AsyncDeepgramClient

        client = AsyncDeepgramClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.self_hosted.v1.distribution_credentials.delete(
                project_id="123456-7890-1234-5678-901234",
                distribution_credentials_id="8b36cfd0-472f-4a21-833f-2d6343c3a2f3",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(
            project_id, distribution_credentials_id, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..types.agent_think_models_v1response_models_item_one_id import AgentThinkModelsV1ResponseModelsItemOneId


class AgentThinkModelsV1ResponseModelsItemOneParams(typing_extensions.TypedDict):
    """
    Anthropic models
    """

    id: AgentThinkModelsV1ResponseModelsItemOneId
    """
    The unique identifier of the Anthropic model
    """

    name: str
    """
    The display name of the model
    """

    provider: typing.Literal["anthropic"]
    """
    The provider of the model
    """
